use bevy::ecs::system::SystemParam;
use bevy::math::Vec2;
use bevy::prelude::{OrthographicProjection, Query, Vec4Swizzles, Window, With};
use bevy::utils::thiserror::Error;
use bevy::window::PrimaryWindow;
use crate::CameraUIKayak;

pub fn unproject_from_physical(
    point: Vec2,
    window: &Window,
    projection: &OrthographicProjection,
) -> Vec2 {
    let projection_area = projection.area;
    let projection_width = projection_area.width();
    let projection_height = projection_area.height();

    let window_width = window.width();
    let window_height = window.height();

    let width_ratio = projection_width / window_width;
    let height_ratio = projection_height / window_height;

    point * Vec2::new(width_ratio, height_ratio)
}

#[derive(SystemParam)]
pub struct UnprojectManager<'w, 's> {
    windows: Query<'w, 's, &'static Window, With<PrimaryWindow>>,
    camera: Query<'w, 's, &'static OrthographicProjection, With<CameraUIKayak>>,
}

#[derive(Debug, Error)]
#[error("Invalid ECS state to perform unprojection")]
pub struct InvalidUnprojection;

impl<'w, 's> UnprojectManager<'w, 's> {
    pub fn unproject(&self, position: Vec2) -> Result<Vec2, InvalidUnprojection> {
        let window = self.windows.get_single().map_err(|_| InvalidUnprojection)?;
        let camera = self.camera.get_single().map_err(|_| InvalidUnprojection)?;

        Ok(unproject_from_physical(
            position,
            window,
            camera,
        ))
    }
}
